###############################################################################
#                                                                             #
#=============================================================================#
#       SCRIPT TO ANALYSE CONTRIBUTION OF ENVIRONMENTAL AND SPATIAL           #
#          FACORTS TO GENETIC DIFFERENTIATION for P. nutans                   #
#=============================================================================#
#                                                                             #
#                                                                             #
###############################################################################

###  modified from a script downloaded in doi:10. 5061/dryad.060d2



### settings

## working directory; you may have to do this or not, depending on the operating system
setwd('/your/path')
# create output directory for results
dir.create('16models_results', recursive = TRUE)

## packages
library(ade4)
library(gstat)
library(introgress)
library(MCMCglmm)
library(raster)
library(reshape2)
library(xlsx)


### load data

## predictors
# environmental data
soildist<-read.csv('pn_soi.csv',row.names = 1)
envdist <- read.csv('pn_bio.csv', row.names = 1)

# spatial data
geodist <- read.csv('pn_geo.csv', row.names = 1)
altdist<-read.csv('pn_alt.csv',row.names = 1)


## responses
# RAD-seq neutral
radn <- read.xlsx('pn_fst_neutral.xlsx', stringsAsFactors = FALSE, sheetIndex = 1, row.names = 1)
# RAD-seq outliers
radp <- read.xlsx('pn_fst_positive.xlsx', stringsAsFactors = FALSE, sheetIndex = 1, row.names = 1)
radg <- read.xlsx('pn_fst_negative.xlsx', stringsAsFactors = FALSE, sheetIndex = 1, row.names = 1)


## creat data tables for Fst modeling
# RAD-seq negative
dat.radg <- read.table("pn_pop.txt",header = T)
dat.radg$gen <- apply(dat.radg, 1, function(x) { radg[x[1],x[2]] })
dat.radg$gen <- as.numeric(dat.radg$gen)
dat.radg$env <- scale(apply(dat.radg, 1, function(x) { envdist[x[1],x[2]] }))
dat.radg$geo <- scale(apply(dat.radg, 1, function(x) { geodist[x[1],x[2]] }))
dat.radg$soi <- scale(apply(dat.radg, 1, function(x) { soildist[x[1],x[2]] }))
dat.radg$alt <- scale(apply(dat.radg, 1, function(x) { altdist[x[1],x[2]] }))
dat.radg$Pop1 <- factor(dat.radg$Pop1)
dat.radg$Pop2 <- factor(dat.radg$Pop2)
#dat.radg$gen[dat.radg$gen<0] <- 0
# RAD-seq neutral
dat.radn <- read.table("pn_pop.txt",header = T)
dat.radn$gen <- apply(dat.radn, 1, function(x) { radn[x[1],x[2]] })
dat.radn$gen <- as.numeric(dat.radn$gen)
dat.radn$env <- scale(apply(dat.radn, 1, function(x) { envdist[x[1],x[2]] }))
dat.radn$geo <- scale(apply(dat.radn, 1, function(x) { geodist[x[1],x[2]] }))
dat.radn$soi <- scale(apply(dat.radn, 1, function(x) { soildist[x[1],x[2]] }))
dat.radn$alt <- scale(apply(dat.radn, 1, function(x) { altdist[x[1],x[2]] }))
dat.radn$Pop1 <- factor(dat.radn$Pop1)
dat.radn$Pop2 <- factor(dat.radn$Pop2)

# RAD-seq outlier
dat.radp <- read.table("pn_pop.txt",header = T)
dat.radp$gen <- apply(dat.radp, 1, function(x) { radp[x[1],x[2]] })
dat.radp$gen <- as.numeric(dat.radp$gen)
dat.radp$env <- scale(apply(dat.radp, 1, function(x) { envdist[x[1],x[2]] }))
dat.radp$geo <- scale(apply(dat.radp, 1, function(x) { geodist[x[1],x[2]] }))
dat.radp$soi <- scale(apply(dat.radp, 1, function(x) { soildist[x[1],x[2]] }))
dat.radp$alt <- scale(apply(dat.radp, 1, function(x) { altdist[x[1],x[2]] }))
dat.radp$Pop1 <- factor(dat.radp$Pop1)
dat.radp$Pop2 <- factor(dat.radp$Pop2)


### Modeling genetic differentiation of neutral and outlier portions of RAD-seq data

## RAD-seq negative
# do models
mod.null <- MCMCglmm(fixed = gen ~ 1, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo <- MCMCglmm(fixed = gen ~ geo, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env <- MCMCglmm(fixed = gen ~ env, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.soi <- MCMCglmm(fixed = gen ~ soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.alt <- MCMCglmm(fixed = gen ~ alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env <- MCMCglmm(fixed = gen ~ geo + env, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_soi <- MCMCglmm(fixed = gen ~ geo + soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_alt <- MCMCglmm(fixed = gen ~ geo + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env_soi <- MCMCglmm(fixed = gen ~ env + soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env_alt <- MCMCglmm(fixed = gen ~ env + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.soi_alt <- MCMCglmm(fixed = gen ~ soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env_soi <- MCMCglmm(fixed = gen ~ geo + env + soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env_alt <- MCMCglmm(fixed = gen ~ geo + env + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_soi_alt <- MCMCglmm(fixed = gen ~ geo + soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env_soi_alt <- MCMCglmm(fixed = gen ~ env + soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env_soi_alt <- MCMCglmm(fixed = gen ~ geo + env + soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radg, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)

# assess DICs
DICs.radg <- data.frame(array(NA, c(16, 3)))
row.names(DICs.radg) <- c('mod.null', 'mod.geo', 'mod.env', 'mod.soi', 'mod.alt', 'mod.geo_env', 'mod.geo_soi','mod.geo_alt','mod.env_soi', 'mod.env_alt', 'mod.soi_alt','mod.geo_env_soi', 'mod.geo_env_alt', 'mod.geo_soi_alt', 'mod.env_soi_alt', 'mod.geo_env_soi_alt')
names(DICs.radg) <- c('DIC', 'deltaDIC', 'DICweight')
DICs.radg[,'DIC'] <- as.matrix(c(mod.null$DIC, mod.geo$DIC, mod.env$DIC, mod.soi$DIC, mod.alt$DIC, mod.geo_env$DIC, mod.geo_soi$DIC, mod.geo_alt$DIC, mod.env_soi$DIC, mod.env_alt$DIC, mod.soi_alt$DIC, mod.geo_env_soi$DIC, mod.geo_env_alt$DIC, mod.geo_soi_alt$DIC, mod.env_soi_alt$DIC, mod.geo_env_soi_alt$DIC))
DICs.radg[,'deltaDIC'] <- with(data.frame(DICs.radg), DIC-min(DIC))
DICs.radg[,'DICweight'] <- with(data.frame(DICs.radg), exp(-deltaDIC / 2) / sum(exp(-deltaDIC / 2)))
# save DIC table
save(DICs.radg, file = '16models_results/DICs_radg')
write.table(DICs.radg,file='16models_results/DICs_radg.txt',quote=F, sep='\t')
# save models
save(list = c('mod.null', 'mod.geo', 'mod.env', 'mod.soi', 'mod.alt', 'mod.geo_env', 'mod.geo_soi','mod.geo_alt','mod.env_soi', 'mod.env_alt', 'mod.soi_alt','mod.geo_env_soi',
              'mod.geo_env_alt', 'mod.geo_soi_alt', 'mod.env_soi_alt', 'mod.geo_env_soi_alt'), file = '16models_results/models_radg')
rm(list = ls(pattern = 'mod\\.'))

## RAD-seq neutral
# do models
mod.null <- MCMCglmm(fixed = gen ~ 1, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo <- MCMCglmm(fixed = gen ~ geo, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env <- MCMCglmm(fixed = gen ~ env, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.soi <- MCMCglmm(fixed = gen ~ soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.alt <- MCMCglmm(fixed = gen ~ alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env <- MCMCglmm(fixed = gen ~ geo + env, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_soi <- MCMCglmm(fixed = gen ~ geo + soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_alt <- MCMCglmm(fixed = gen ~ geo + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env_soi <- MCMCglmm(fixed = gen ~ env + soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env_alt <- MCMCglmm(fixed = gen ~ env + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.soi_alt <- MCMCglmm(fixed = gen ~ soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env_soi <- MCMCglmm(fixed = gen ~ geo + env + soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env_alt <- MCMCglmm(fixed = gen ~ geo + env + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_soi_alt <- MCMCglmm(fixed = gen ~ geo + soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env_soi_alt <- MCMCglmm(fixed = gen ~ env + soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env_soi_alt <- MCMCglmm(fixed = gen ~ geo + env + soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radn, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)

# assess DICs
DICs.radn <- data.frame(array(NA, c(16, 3)))
row.names(DICs.radn) <- c('mod.null', 'mod.geo', 'mod.env', 'mod.soi', 'mod.alt', 'mod.geo_env', 'mod.geo_soi','mod.geo_alt','mod.env_soi', 'mod.env_alt', 'mod.soi_alt','mod.geo_env_soi',
                          'mod.geo_env_alt', 'mod.geo_soi_alt', 'mod.env_soi_alt', 'mod.geo_env_soi_alt')
names(DICs.radn) <- c('DIC', 'deltaDIC', 'DICweight')
DICs.radn[,'DIC'] <- c(mod.null$DIC, mod.geo$DIC, mod.env$DIC, mod.soi$DIC, mod.alt$DIC, mod.geo_env$DIC, mod.geo_soi$DIC, mod.geo_alt$DIC, mod.env_soi$DIC, mod.env_alt$DIC, mod.soi_alt$DIC, mod.geo_env_soi$DIC,
                       mod.geo_env_alt$DIC, mod.geo_soi_alt$DIC, mod.env_soi_alt$DIC, mod.geo_env_soi_alt$DIC)
DICs.radn[,'deltaDIC'] <- with(data.frame(DICs.radn), DIC-min(DIC))
DICs.radn[,'DICweight'] <- with(data.frame(DICs.radn), exp(-deltaDIC / 2) / sum(exp(-deltaDIC / 2)))
# save DIC table
save(DICs.radn, file = '16models_results/DICs_radn')
write.table(DICs.radn,file='16models_results/DICs_radn.txt',quote=F, sep='\t')
# save models
save(list = c('mod.null', 'mod.geo', 'mod.env', 'mod.soi', 'mod.alt', 'mod.geo_env', 'mod.geo_soi','mod.geo_alt','mod.env_soi', 'mod.env_alt', 'mod.soi_alt','mod.geo_env_soi',
              'mod.geo_env_alt', 'mod.geo_soi_alt', 'mod.env_soi_alt', 'mod.geo_env_soi_alt'), file = '16models_results/models_radn')
rm(list = ls(pattern = 'mod\\.'))

## RAD-seq positive
# do models
mod.null <- MCMCglmm(fixed = gen ~ 1, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo <- MCMCglmm(fixed = gen ~ geo, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env <- MCMCglmm(fixed = gen ~ env, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.soi <- MCMCglmm(fixed = gen ~ soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.alt <- MCMCglmm(fixed = gen ~ alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env <- MCMCglmm(fixed = gen ~ geo + env, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_soi <- MCMCglmm(fixed = gen ~ geo + soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_alt <- MCMCglmm(fixed = gen ~ geo + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env_soi <- MCMCglmm(fixed = gen ~ env + soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env_alt <- MCMCglmm(fixed = gen ~ env + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.soi_alt <- MCMCglmm(fixed = gen ~ soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env_soi <- MCMCglmm(fixed = gen ~ geo + env + soi, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env_alt <- MCMCglmm(fixed = gen ~ geo + env + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_soi_alt <- MCMCglmm(fixed = gen ~ geo + soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.env_soi_alt <- MCMCglmm(fixed = gen ~ env + soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)
mod.geo_env_soi_alt <- MCMCglmm(fixed = gen ~ geo + env + soi + alt, random = ~ idv(mult.memb( ~ Pop1 + Pop2)), data = dat.radp, pr = TRUE, nitt = 10000000, burnin = 2000000, thin = 1000, verbose = FALSE)

# assess DICs
DICs.radp <- data.frame(array(NA, c(16, 3)))
row.names(DICs.radp) <- c('mod.null', 'mod.geo', 'mod.env', 'mod.soi', 'mod.alt', 'mod.geo_env', 'mod.geo_soi','mod.geo_alt','mod.env_soi', 'mod.env_alt', 'mod.soi_alt','mod.geo_env_soi',
                          'mod.geo_env_alt', 'mod.geo_soi_alt', 'mod.env_soi_alt', 'mod.geo_env_soi_alt')
names(DICs.radp) <- c('DIC', 'deltaDIC', 'DICweight')
DICs.radp[,'DIC'] <- c(mod.null$DIC, mod.geo$DIC, mod.env$DIC, mod.soi$DIC, mod.alt$DIC, mod.geo_env$DIC, mod.geo_soi$DIC, mod.geo_alt$DIC, mod.env_soi$DIC, mod.env_alt$DIC, mod.soi_alt$DIC, mod.geo_env_soi$DIC,
                       mod.geo_env_alt$DIC, mod.geo_soi_alt$DIC, mod.env_soi_alt$DIC, mod.geo_env_soi_alt$DIC)
DICs.radp[,'deltaDIC'] <- with(data.frame(DICs.radp), DIC-min(DIC))
DICs.radp[,'DICweight'] <- with(data.frame(DICs.radp), exp(-deltaDIC / 2) / sum(exp(-deltaDIC / 2)))
# save DIC table
save(DICs.radp, file = '16models_results/DICs_radp')
write.table(DICs.radp,file='16models_results/DICs_radp.txt',quote=F, sep='\t')
# save models
save(list = c('mod.null', 'mod.geo', 'mod.env', 'mod.soi', 'mod.alt', 'mod.geo_env', 'mod.geo_soi','mod.geo_alt','mod.env_soi', 'mod.env_alt', 'mod.soi_alt','mod.geo_env_soi',
              'mod.geo_env_alt', 'mod.geo_soi_alt', 'mod.env_soi_alt', 'mod.geo_env_soi_alt'), file = '16models_results/models_radp')
rm(list = ls(pattern = 'mod\\.'))


